#include <assert.h>
#include <stdlib.h>
#include "async.h"
#include "prot-a-LyStr.h"
#include "connection.h"

Async_storage::Async_storage(Connection *c)
  : port(c), pending_async()
{
    for (int i=0; i < MAX_ASYNC; i++)
	admin_table[i] = NULL;
}

Status 
Async_storage::handle_async(int this_many)
{
    while (this_many-- > 0 && !pending_async.empty())
    {
	Pending next = pending_async.remove_front();
	admin_table[next.type]->handle_message(next.object);
    }

    return pending_async.empty() ? st_ok : st_pending;
}

Connection *
Async_storage::connection() const
{
    return port;
}

void 
Async_storage::register_administrator(Async_name n, 
				      Async_administrator* adm)
{
    assert(0 <= n);
    assert(n < MAX_ASYNC);
    assert(admin_table[n] == NULL);
    admin_table[n] = adm;
    if (!is_registered)
    {
	port->register_async((void*)this, Async_storage::parse_and_store);
	is_registered = TRUE;
    }
}

void 
Async_storage::parse_and_store(void *async_storage,
			       prot_a_LyStr async_msg)
{
    Async_storage *ast = (Async_storage *)async_storage;

    async_msg.stol();			// Skip 'number of items' field.
    long type = async_msg.stol();

    if (type >= 0 && type < MAX_ASYNC && ast->admin_table[type] != NULL)
    {
	Pending foo;   
	foo.object = ast->admin_table[type]->parse(async_msg);
	foo.type = type;
	ast->pending_async.append(foo);
    }
}

Async_administrator::Async_administrator(Async_storage *ast)
  : handlers(NULL), storage(ast), is_registered(FALSE)
{
}

virtual 
Async_administrator::~Async_administrator()
{
    assert(handlers==NULL);
}

void* 
Async_administrator::register_handler(void* userdata, 
				      void* handler,
				      Async_name name)
{
    handlers = new Hinfo(handler, userdata, handlers);
    if (!is_registered)
    {
	storage->register_administrator(name, this);
	is_registered = TRUE;
    }

    return handlers;
}

void 
Async_administrator::unregister_handler(void* unwanted)
{
    for (Hinfo **ptr = &handlers; *ptr != NULL; ptr = &(*ptr)->next)
    {
	if (*ptr == unwanted)
	{
	    Hinfo *del = *ptr;
	    *ptr = (*ptr)->next;
	    delete del;
	    return;
	}
    }
    
    abort();
}

Async_administrator::Hinfo::Hinfo(void *hhandler,
				  void *uuserdata, 
				  Hinfo *nnext)
  : userdata(uuserdata), handle(hhandler), next(nnext)
{
}

Async_administrator::Hinfo *
Async_administrator::handler_list_head() const
{
    return handlers;
}

Async_text_created::Async_text_created()
: tstat()
{
}

//Async_text_created::Async_text_created(const Async_text_created &t)
//: tstat(t.tstat)
//{
//}

Async_text_created::~Async_text_created()
{
}

Async_text_created &
Async_text_created::operator =(const Async_text_created &t)
{
    tstat = t.tstat;
    return *this;
}

Text_stat 
Async_text_created::text_stat() const
{
    return tstat;
}

Async_text_created::Async_text_created(prot_a_LyStr &p)
{
    Text_no tno = p.stol();
    Text_stat t(tno, p);
    tstat = t;
}

Async_text_created_administrator::
Async_text_created_administrator(Async_storage *as)
  : Async_administrator(as)
{
}

Async_text_created_administrator::~Async_text_created_administrator()
{
}

void* 
Async_text_created_administrator::register_handler(void *userdata, 
						   Handler* hnd)
{
    return Async_administrator::register_handler(userdata, (void*)hnd, 
						 ASYNC_NEW_TEXT);
}


void  
Async_text_created_administrator::unregister_handler(void* hnd)
{
    Async_administrator::unregister_handler(hnd);
}

const Async_message* 
Async_text_created_administrator::parse(prot_a_LyStr &msg) const
{
    return new Async_text_created(msg);
}

void 
Async_text_created_administrator::handle_message(const Async_message *msg)
{
    for (struct Hinfo* ptr = handler_list_head();
	 ptr != NULL;
	 ptr = ptr->next)
    {
	Async_text_created *m = (Async_text_created*)msg;
	(*(Handler*)ptr->handle)(ptr->userdata, *m);
    }
}
