#include <assert.h>
#include <iostream.h>
#include "question.h"
#include "refno.h"
#include "connection.h"
#include <stdlib.h>

// refno stuff (se end of file for question stuff).

int limit=0;
const CHUNKSIZ = 5;

static question **map = NULL;

Ref_no::Ref_no()
{
    if (map == NULL)
    {
	limit = CHUNKSIZ;
	map = new (question *[limit]);
    }

    for (int i=0; i < limit; i++)
	if (map[i] == NULL)
	{
	    ref = i;
	    return;
	}

    question **newmap = new (question *[limit + CHUNKSIZ]);
    for (i=0; i < limit; i++)
	newmap[i] = map[i];
    ref = limit;
    limit += CHUNKSIZ;
    delete []map;
    map = newmap;
}

Ref_no::~Ref_no()
{
    map[ref] = NULL;
}

int Ref_no::ref_no() const
{
    return ref;
}
    
question *
lookup_refno(int r)
{
    assert(0 <= r && r < limit);
    return map[r];
}

// question stuff

question::question(Connection *conn)
    : port(conn)
{
    refno = new Ref_no;
    map[refno->ref_no()] = this;
    state = st_pending;
}

question::~question()
{
    if (refno)
    {
	cerr << "Warning: destructing unanswered question with ref_no=" 
	     << refno->ref_no() << endl;
	deallocate_refno();
    }
}

Status
question::status()
{
    if (state == st_pending)
    {
	if (Connection::drain(0) == st_error)
	{
	    errcode = KOM_NO_CONNECT;
	    state = st_error;
	}
    }

    return state;
}

Kom_err
question::error()
{
    return errcode;
}

void
question::receive_err(int err)
{
    state = st_error;
    errcode = err;
    deallocate_refno();
}

Status question::receive()
{
    while (state == st_pending)
    {
	if (Connection::drain(100) == st_error)
	{
	    errcode = KOM_NO_CONNECT;
	    state = st_error;
	}
    }

    return state;
}

void
question::deallocate_refno()
{
    delete refno;
    refno = NULL;
}

int
question::ref_no() const
{
    assert(refno != NULL);
    return refno->ref_no();
}
