#include <stddef.h>
#include <iostream.h>

#include "session.h"

#include "login_q.h"
#include "get_time_q.h"
#include "lookup_name_q.h"
#include "re_lookup_person_q.h"
#include "get_marks_q.h"


// ================================================================ 
//                    Constructors and Destructors                  


Session::Session(Connection *connection)
    : conn(connection), all_confs(NULL), all_persons(NULL), 
      all_maps(NULL), all_texts(NULL)
{
}
 
Session::Session(const Session &s)
    : conn(s.conn), all_confs(s.all_confs), all_persons(s.all_persons), 
      all_maps(s.all_maps), all_texts(s.all_texts)
{
}


Session::~Session()
{
    if (all_confs != NULL)
	delete all_confs;
    if (all_persons != NULL)
	delete all_persons;
    if (all_maps != NULL)
	delete all_maps;
    if (all_texts != NULL)
	delete all_texts;
}


// ================================================================ 
//

const Session & 
Session::operator = (const Session &s)
{
    conn    = s.conn;
    all_confs   = s.all_confs;
    all_persons = s.all_persons;
    all_maps    = s.all_maps;
    all_texts   = s.all_texts;

    return *this;
}


ostream &
operator <<(ostream &os, const Session &s)
{
    cout << "A session object";
    return os;
}


void
Session::set_connection(Connection *connection)
{
    conn = connection;
    //+++ Reset confs, persons, maps and texts here.
}


// ================================================================


Conferences & 
Session::conferences()
{
    assert(conn != NULL);

    if (all_confs == NULL) {
	all_confs = new Conferences(conn);
    }

    return *all_confs;
}


Persons &
Session::persons()
{
    if (all_persons == NULL) {
	all_persons = new Persons(conferences());
    }

    return *all_persons;
}


Maps &
Session::maps()
{
    if (all_maps == NULL) {
	all_maps = new Maps(conferences());
    }

    return *all_maps;
}


Texts &  
Session::texts()
{
    assert(conn != NULL);

    if (all_texts == NULL) {
	all_texts = new Texts(conn);
    }

    return *all_texts;

}


// ----------------------------------------------------------------


Status
Session::login(Pers_no person, LyStr &passwd, Bool invisible)
{
    login_question in(conn, person, passwd, invisible);

    // +++ Delete all caches!  Should this be a separate method?
    return in.receive();
}


struct tm
Session::get_time()
{
    get_time_question   timeq(conn);
    Status              stat = timeq.receive();

    if (stat == st_ok)
	return timeq.result();

    // +++ Error handling!!
    return timeq.result();
}


Array<Micro_conf>
Session::lookup_name(const LyStr &pattern)
{
    lookup_name_question   lookupq(conn, pattern);

    if (lookupq.receive() == st_ok) 
	return lookupq.result();
    else
	abort();		// +++Error handling!
}


Array<Pers_no>
Session::regexp_lookup_persons(const LyStr &pattern)
{
    re_lookup_person_question   lookupq(conn, pattern);

    if (lookupq.receive() == st_ok) 
	return lookupq.result();
    else
	abort();		// +++Error handling!
}


Array<Mark>
Session::get_marks()
{
    get_marks_question  markq(conn);

    if (markq.receive() == st_ok) 
	return markq.result();
    else
	abort();		// +++Error handling!
}
