#include <iostream.h>

#include "text.h"
#include "LyStr.h"
#include "array.h"
#include "person.h"
#include "conference.h"

#include "get_text_q.h"
#include "mark_as_read_q.h"
#include "mark_text_q.h"
#include "unmark_text_q.h"


// ================================================================
// ================================================================
//                            Class Text


// ================================================================ 
//                    Constructors and Destructors                  


Text::Text()
{
}


Text::Text(Text_no t, 
	   Text_stat_cache *t_cache)
    : tno(t), cache(t_cache)
{
}
 
Text::Text(const Text &t)
    : tno(t.tno), cache(t.cache)
{
}


Text::~Text()
{
}


// ================================================================ 
//

const Text & 
Text::operator = (const Text &t)
{
    tno = t.tno;
    cache = t.cache;
    return *this;
}

Status
Text::prefetch() const
{
    return cache->prefetch(tno);
}

Bool
Text::valid() const
{
    return cache->get(tno).valid();
}

int
Text::errno() const
{
    return cache->get(tno).errno();
}

Text_no
Text::text_no() const
{
    return tno;
}

int
Text::prefetch_for_display(int max_questions, Session *sess) const
{
    // +++ Not yet implemented.
    return 0;
}

struct tm
Text::creation_time() const
{
    assert(valid());
    return cache->get(tno).creation_time();
}

Pers_no
Text::author() const
{
    assert(valid());
    return cache->get(tno).author();
}

long       
Text::length() const
{
    assert(valid());
    return cache->get(tno).length();
}

int         
Text::num_marks() const
{
    assert(valid());
    return cache->get(tno).num_marks();
}

int
Text::num_recipients() const
{
    assert(valid());
    return cache->get(tno).num_recipients();
}

Recipient* 
Text::recipient(int i) const
{
    assert(valid());
    return cache->get(tno).recipient(i);
}

int
Text::num_comment_in() const
{
    assert(valid());
    return cache->get(tno).num_comment_in();
}

Text_no	
Text::comment_in(int i) const
{
    assert(valid());
    return cache->get(tno).comment_in(i);
}

int
Text::num_footnote_in() const
{
    assert(valid());
    return cache->get(tno).num_footnote_in();
}

Text_no
Text::footnote_in(int i) const
{
    assert(valid());
    return cache->get(tno).footnote_in(i);
}

int		
Text::num_uplinks() const
{
    assert(valid());
    return cache->get(tno).num_uplinks();
}

Uplink*
Text::uplink(int i)	 const
{
    assert(valid());
    return cache->get(tno).uplink(i);
}

LyStr
Text::text_mass(long start=0, long end=LONG_MAX)
{
    get_text_question q(cache->connection(), tno, start, end);
    if (q.receive() == st_ok)
	return q.result();
    else
	return "Du f}r inte l{sa texten";    // FIXME! Error handling...
}


Status
Text::mark_as_read(Conf_no cno)
{
    Text_stat   textstat;
    int         i;

    assert(valid());
    textstat = cache->get(tno);
    
    // If cno=0, then mark the text as read in all conferences.
    if (cno == 0) {
	Array<mark_as_read_question *>   mark_q_arr(textstat.num_recipients());
	Status                           return_status;

	for (i = 0; i < textstat.num_recipients(); ++i) {
	    mark_q_arr[i] 
		= new mark_as_read_question(cache->connection(),
					    textstat.recipient(i)->recipient(),
					    textstat.recipient(i)->local_no());
	}

	return_status = st_ok;
	for (i = 0; i < textstat.num_recipients(); ++i) {
	    if (return_status == st_ok)
		return_status = mark_q_arr[i]->receive();
	    else
		(void) mark_q_arr[i]->receive();
	}
	return return_status;

    } else {
	// Search the recipients for the one to mark as read.
	for (i = 0; i < textstat.num_recipients(); ++i) {
	    mark_as_read_question  * mark_q;
	    Recipient              * recip;

	    recip = textstat.recipient(i);
	    if (recip->recipient() == cno) {
		mark_q = new mark_as_read_question(cache->connection(),
						   recip->recipient(),
						   recip->local_no());
		return mark_q->receive();
	    }
	}
    }

    return st_ok;
}



Status
Text::mark(unsigned char mark_type)
{
    // +++FIXME: This loses the error number if it fails. Bad, bad.
    // This comment also applies to Text::unmark, and probably other
    // methods as well.
    mark_text_question   mark_q(cache->connection(), tno, mark_type);
    return mark_q.receive();
}

Status
Text::unmark()
{
    unmark_text_question   unmark_q(cache->connection(), tno);
    return unmark_q.receive();
}

// ================================================================
// ================================================================
//                            Class Texts


// ================================================================ 
//                    Constructors and Destructors                  


Texts::Texts()
    : connection(NULL), ts_cache(NULL)
{
}
Texts::Texts(Connection *conn)
    : connection(conn), ts_cache(new Text_stat_cache(conn))
{
}
 

Texts::Texts(const Texts &text)
{
    abort();
}


Texts::~Texts()
{
    if (ts_cache)
	delete ts_cache;
}


// ================================================================ 
//

const Texts & 
Texts::operator = (const Texts &)
{
    cerr << "Operator= for class Texts is not yet implemented\n";
    abort();
}

void
Texts::set_connection(Connection *conn)
{
    assert(connection == NULL);
    connection = conn;
    ts_cache = new Text_stat_cache(conn);
}

Text
Texts::operator [](Text_no textno)
{
    assert(ts_cache != NULL);
    return Text(textno, ts_cache);
}

Text_stat_cache *
Texts::read_text_stat_cache() const
{
    return ts_cache;
}

