#ifndef TEXT__
#define TEXT__

#include <limits.h>
#include "text-cache.h"
#include "LyStr.h"


class Session;
class Connection;

class Text {
  public:
    Text();
    Text(Text_no, Text_stat_cache *);
    Text(const Text &);
    ~Text();

    const Text & operator = (const Text &);

    Status   	prefetch() 	 const;
    Bool	valid() 	 const;
    int		errno()		 const;
    Text_no     text_no()        const;

    // Prefetch everything that is needed by nilkom to display this
    // text, but issue at most max_questions questions. Return the
    // number of pending questions, or -1 if a hard error occurs.
    int		prefetch_for_display(int max_questions, 
				     Session *sess) const;

    struct tm   creation_time()  const;
    Pers_no	author() 	 const;
    long        length()         const;
    int         num_marks()      const;
    int         num_recipients() const;
    Recipient*  recipient(int)   const;
    int		num_comment_in() const;
    Text_no	comment_in(int)	 const;
    int		num_footnote_in()const;
    Text_no	footnote_in(int) const;
    int		num_uplinks()	 const;
    Uplink*	uplink(int)	 const;

    LyStr	text_mass(long start=0, long end=LONG_MAX);

    Status      mark_as_read(Conf_no confno = 0);
    Status      mark(unsigned char mark_type = 255);
    Status      unmark();

  private:
    Text_no tno;
    Text_stat_cache *cache;
};



class Texts {
  public:
    Texts();
    Texts(Connection *);
    Texts(const Texts &);
    ~Texts();

    const Texts & operator = (const Texts &);
    void set_connection(Connection *);

    Text  operator[](Text_no);

    Text_stat_cache * read_text_stat_cache() const;

  private:
    Connection      * connection;
    Text_stat_cache * ts_cache;
};


#endif  // TEXT__
