# Misc. TCL code for nilkom

# global variables (shared)
#  review_list   - list of textnos to review

proc read_text {tno} {
    global to_mark_as_read

    print_text $tno
    lappend to_mark_as_read $tno
}

proc k {{parent senaste}} {
	global inhibit_prompt
	global senaste

	if {[string compare $parent senaste] == 0} {
            start_comment $senaste
	} else {
            start_comment $parent
	}
	set inhibit_prompt 1
}
	
proc next_conference {} {
	print [format "Du hamnade i %s\n" \
		 [kom_conference name [kom_unread goto_next_conf]]]
}

proc reject {} {
	global to_mark_as_read

	print "Text $to_mark_as_read lsmarkeras ej.\n"
	set to_mark_as_read {}
}

proc quick_unread {} {
    set total 0

    print "Du har olsta inlgg i fljande mten:\n\n"
    foreach conf [kom_unread unread_texts] {
        set confno [lindex $conf 0]
        set min [lindex $conf 1]
        set max [lindex $conf 2]

	if {$max == $min} {
	    print [format "     %6d %s\n" $min \
		[kom_conference name $confno]]
	} else {
	    print [format "%4d-%6d %s\n" $min $max \
		[kom_conference name $confno]]
	}
	incr total $min
    }
    print "\nTotalt minst $total olsta\n\n\n"
}

proc unread {} {
    set unread_confs [llength [kom_unread unread_texts]]
    set neednewline 0
    while {[kom_unread prefetch_map 0 20 $unread_confs 70 0] > 0} {
    }
    kom_unread prefetch_map 0 0 0 10 10
    quick_unread
}

proc keypress {n} {
    global keyhandler more_row

    set more_row 0
    $keyhandler $n
}

set prefetch_state 0

proc timeout {} {
    global prefetch_state

    switch $prefetch_state {
	0
	    {
    		if {[kom_unread prefetch_map 1000 1000 \
			 [llength [kom_unread unread_texts]] 2 0] == 0} {
		    set prefetch_state 1
		}
	    }
	1
	    {
		kom_unread prefetch 3 0 0 2 0
	    }
    }

    return 0
}

proc trim_review_list {} {
    global review_list

    set f 1
    while {$f==1} {
	set f 0
	if {[llength $review_list] > 0} {
	    safe_eval {kom_text author [lindex $review_list 0]} {
		{
		    {KOM BADTEXT}
		    {
			set review_list [lrange $review_list 1 end]
			set f 1
		    }
		}
	    }
	}
    }
}

proc review_next {} {
    global review_list

    if {[llength $review_list] == 0} {
	print {Det finns inget att se hr.\n}
    } else {
	print_text [lindex $review_list 0]
	set review_list [lrange $review_list 1 end]
    }
}

proc review_commented {tno} {
    global review_list

    set l {}
    set i 0
    while {$i<[kom_text num_uplinks $tno]} {
	lappend l [uplink parent [kom_text uplink $tno $i]]
	incr i
    }

    set review_list "$l $review_list"
    trim_review_list
    review_next
}

proc review_all_comments {tno} {
    global review_list

    set l {}

    set i 0
    while {$i<[kom_text num_footnotes $tno]} {
	lappend l [kom_text footnote $tno $i]
	incr i
    }

    set i 0
    while {$i<[kom_text num_comments $tno]} {
	lappend l [kom_text comment $tno $i]
	incr i
    }

    set review_list "$l $review_list"
    trim_review_list
    review_next
}

proc mark {tno type} {
    safe_eval {
	kom_text mark $tno $type
	print "Text $tno markerad med markeringstyp $type.\n"
    } {
	{{KOM BADTEXT} {print "Det finns ingen sdan text.\n"}}
    }
}

proc help {} {
    global nilkom_version

    print {Vlkommen till nilkom!\n}
    print {\n}
    print {Trycker RETURN fr att utfra defaultkommandot.\n}
    print {Tryck Ctr-C fr att avsluta nilkom.\n}
    print {Skriv "lista kommandon" fr att se vad du kan gra.\n}
    print {Skicka buggrapporter till ceder@lysator.liu.se.\n}
    print {Ha tlamod med att mnga finesser och kommandon saknas.\n}
    print "Detta r nilkom $nilkom_version\n"
}
