#ifndef TCL_SUPPORT_H
#define TCL_SUPPORT_H

#include <tcl.h> 


// ================================================================
//          Types for parsing arguments to tcl procedures


enum Tclargtype {
    TCLARG_INT,			// An integer argument
    TCLARG_FLOAT,		// A floating point argument
    TCLARG_STRING,		// A string argument
    TCLARG_ARGS,		// An unspecified number of arguments
				// (they are not checked or assigned)
};

const int TCL_ARGS_MAX = 8;

struct Tcl_Args {
    char        * option;
    int           optionnum;
    int           num_args;	// Number of arguments.
    int		  num_optional;	// Number of additional optional arguments.
    Tclargtype    argtypes[TCL_ARGS_MAX];
};


// ================================================================


class Parse_tcl_args {
  public:
    Parse_tcl_args(Tcl_Interp *, Tcl_Args *, int args, char **argv);
    ~Parse_tcl_args();
    
    int option() const;

    long intarg(int) const;
    double floatarg(int) const;
    // Note: the pointers returned by these methods are equal to the
    // pointers that are given to the constructor in the argv
    // argument. That is, they are not allocated by this class.
    char *stringarg(int) const;

    enum { TCL_MAX_INTS=5, TCL_MAX_FLOATS=4, TCL_MAX_STRINGS=4 };
  private:
    int    tcl_option;

    long     tcl_intargs[TCL_MAX_INTS];
    double   tcl_floatargs[TCL_MAX_FLOATS];
    char   * tcl_stringargs[TCL_MAX_STRINGS];

    int      int_index;
    int      float_index;
    int      string_index;
    int	     parse_tcl_arg(Tcl_Interp *interp, Tclargtype argtype, char *arg);
};

#endif  // TCL_SUPPORT_H
